/// <reference types="express" />
/// <reference types="multer" />
import { Express } from "express";
import * as express from "express";
import * as nunjucks from "nunjucks";
import "reflect-metadata";
import * as multer from "multer";
import Config from "./config";
import BaseModule from "./base.module";
import User from "./entities/user.entity";
declare global  {
    interface Array<T> {
        serialize(): Array<any>;
        removeHiddenField(field: string): void;
        addHiddenField(field: string): void;
    }
}
/**
 * Utility function to check if we are in the production environment.
 * @return true if the NODE_ENV is set to "production", false otherwise
 */
export declare function isProduction(): boolean;
/**
 * The App class contains the initialization code for a Lynx application.
 */
export default class App {
    express: Express;
    private _config;
    private _nunjucksEnvironment;
    private _upload;
    private _templateMap;
    readonly config: Config;
    readonly templateMap: any;
    readonly nunjucksEnvironment: nunjucks.Environment;
    readonly upload: multer.Instance;
    constructor(config: Config, modules?: BaseModule[]);
    private recursiveGenerateTemplateMap(path, currentPath);
    private generateTemplateMap(paths);
    private loadTranslations(paths);
    private loadMiddlewares(path);
    private loadControllers(path);
    startServer(port: number): void;
    route(name: string, parameters?: any): string;
    translate(str: string, req: express.Request): string;
    generateTokenForUser(user: User): string;
}
