"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var render_response_1 = require("./render.response");
var redirect_response_1 = require("./redirect.response");
var unauthorized_response_1 = require("./unauthorized.response");
var file_response_1 = require("./file.response");
var media_entity_1 = require("./entities/media.entity");
var nodemailer_1 = require("nodemailer");
var mailClient;
var guard = false;
function syncronizedInit() {
    if (guard)
        return;
    guard = true;
    if (!mailClient) {
        try {
            nodemailer_1.createTestAccount(function (err, account) {
                if (err) {
                    console.log(err);
                    guard = false;
                    return;
                }
                mailClient = nodemailer_1.createTransport({
                    host: "smtp.ethereal.email",
                    port: 587,
                    secure: false,
                    auth: {
                        user: account.user,
                        pass: account.pass // generated ethereal password
                    }
                });
                guard = false;
            });
        }
        catch (e) {
            guard = false;
            console.log(e);
        }
    }
}
/**
 * This class defines the basic class for any controllers. It implements a lot
 * of utility methods in order to correctly generate any response.
 */
var BaseController = /** @class */ (function () {
    function BaseController(app) {
        this.app = app;
        syncronizedInit();
    }
    Object.defineProperty(BaseController.prototype, "metadata", {
        get: function () {
            return this._metadata;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * This method is called only when the constructed has been completed.
     * Since this method is async, it can be used to perform some initialization
     * that needed the use of the await keyword.  */
    BaseController.prototype.postConstructor = function () {
        return __awaiter(this, void 0, void 0, function () { return __generator(this, function (_a) {
            return [2 /*return*/];
        }); });
    };
    /**
     * This method generate an url to a route starting from the route name and
     * optionally its parameters.
     * If a parameter not is used to generate the route url, it will be appended
     * as a query parameter.
     * @param name the name of the route
     * @param parameters a plain object containing the paramters for the route.
     */
    BaseController.prototype.route = function (name, parameters) {
        return this.app.route(name, parameters);
    };
    /**
     * Generate a web page starting from a template and using a generated context.
     * @param view the name of the view
     * @param req the request object
     * @param context a plain object containing any necessary data needed by the view
     */
    BaseController.prototype.render = function (view, req, context) {
        if (!view.endsWith(".njk")) {
            view = view + ".njk";
        }
        if (!context) {
            context = {};
        }
        context.req = req;
        return new render_response_1.default(view, context);
    };
    /**
     * Redirect the current route to another
     * @param routeName the new of the target route
     */
    BaseController.prototype.redirect = function (routeName) {
        return new redirect_response_1.default(this.route(routeName));
    };
    /**
     * Generate a response suitable to file download. This method can also be
     * used to generate images of specific dimensions.
     * @param path the string path of the file, or a Media object to be downloaded
     * @param options options to correctly generate the output file
     */
    BaseController.prototype.download = function (path, options) {
        if (path instanceof media_entity_1.default) {
            if (path.isDirectory) {
                throw new Error("unable to downlaod a directory");
            }
            var f = new file_response_1.default(path.path);
            f.contentType = path.mimetype;
            if (options) {
                f.options = options;
            }
            return f;
        }
        return new file_response_1.default(path);
    };
    /**
     * Generate an unauthorized response.
     */
    BaseController.prototype.unauthorized = function () {
        return new unauthorized_response_1.default();
    };
    /**
     * Utility method to send an email from a controller. This method is async,
     * so use the await keyword (or eventually a promise) to correctly read the
     * return value.
     * This method uses the template engine to compile the email.
     * @param req the current request
     * @param dest the email destination (can also be an array of addresses)
     * @param subjectTemplateString the subject of the email, that can also be a string template
     * @param textTemplate the text version of the email, referencing a path in the view folders
     * @param htmlTemplate the html version of the email, referencing a path in the view folders
     * @param context a plain object containing any necessary data needed by the view
     */
    BaseController.prototype.sendMail = function (req, dest, subjectTemplateString, textTemplate, htmlTemplate, context) {
        return __awaiter(this, void 0, void 0, function () {
            var subject, text, html, mailOptions, result, e_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!context) {
                            context = {};
                        }
                        context.req = req;
                        subject = this.app.nunjucksEnvironment.renderString(subjectTemplateString, context);
                        if (!textTemplate.endsWith(".njk")) {
                            textTemplate += ".njk";
                        }
                        if (!htmlTemplate.endsWith(".njk")) {
                            htmlTemplate += ".njk";
                        }
                        text = this.app.nunjucksEnvironment.render(textTemplate, context);
                        html = this.app.nunjucksEnvironment.render(htmlTemplate, context);
                        mailOptions = {
                            from: this.app.config.mailer.sender,
                            to: dest,
                            subject: subject,
                            text: text,
                            html: html // html body
                        };
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, mailClient.sendMail(mailOptions)];
                    case 2:
                        result = _a.sent();
                        if (result) {
                            console.log("Preview URL: %s", nodemailer_1.getTestMessageUrl(result));
                            return [2 /*return*/, true];
                        }
                        return [3 /*break*/, 4];
                    case 3:
                        e_1 = _a.sent();
                        console.log(e_1);
                        return [3 /*break*/, 4];
                    case 4: return [2 /*return*/, false];
                }
            });
        });
    };
    /**
     * Utility method to obtain a translated string.
     * @param str the string key to be transalted
     * @param req the original request
     */
    BaseController.prototype.tr = function (str, req) {
        return this.app.translate(str, req);
    };
    return BaseController;
}());
exports.BaseController = BaseController;
