export default interface Config {
    disabledDb: boolean;
    db: {
        type: string;
        host: string;
        port: number;
        username: string;
        password: string;
        database: string;
        entities: string[];
        synchronize: boolean;
        logging: boolean;
    };
    publicFolders: string[];
    viewFolders: string[];
    translationFolders: string[];
    middlewaresFolders: string[];
    controllersFolders: string[];
    sessionSecret: string;
    sessionStore?: any;
    tokenSecret: string;
    mailer: {
        sender: string;
    };
    defaultLanguage: string;
    uploadPath: string;
}
export declare class ConfigBuilder {
    private config;
    constructor(basePath: string);
    setPublicFolders(folders: string[]): ConfigBuilder;
    setViewFolders(folders: string[]): ConfigBuilder;
    setTranslationFolders(folders: string[]): ConfigBuilder;
    setMiddlewaresFolders(folders: string[]): ConfigBuilder;
    setControllersFolders(folders: string[]): ConfigBuilder;
    setSessionSecret(secret: string): ConfigBuilder;
    setTokenSecret(secret: string): ConfigBuilder;
    setSessionStore(store: any): ConfigBuilder;
    setUploadPath(path: string): ConfigBuilder;
    setDefaultLanguage(language: string): ConfigBuilder;
    setEntitiesFolders(folders: string[]): ConfigBuilder;
    setDatabaseType(type: string): ConfigBuilder;
    setDatabaseHost(host: string): ConfigBuilder;
    setDatabasePort(port: number): ConfigBuilder;
    setDatabaseLogin(username: string, password: string): ConfigBuilder;
    setDatabase(database: string): ConfigBuilder;
    disableDB(): ConfigBuilder;
    setMailerSender(address: string): ConfigBuilder;
    build(): Config;
}
