import * as Joi from "joi";
/**
 * Contains the name of the error and a localized error message.
 */
export interface ValidationError {
    name: string;
    message: string;
}
/**
 * This class is used to validate an object using a given schema.
 * It is used by Lynx to automatically validate the body of any requests, using
 * the Body decodator.
 */
export declare class ValidateObject<T> {
    private _obj;
    private schema;
    private valid;
    constructor(obj: any, schema: Joi.Schema, locales: string[]);
    private validate(options);
    /**
     * Verify that the object rescpect the schema.
     * @return true if the object is valid, false otherwise.
     */
    readonly isValid: boolean;
    /**
     * Unwrap the object (can be valid or not!)
     * @return the unwrapped object
     */
    readonly obj: T;
    /**
     * Getter that returns an array of validation errors.
     * @return an array of validation errors. It can not be null.
     */
    readonly errors: ValidationError[];
}
