"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Joi = require("joi");
var itErrors = require("./locale/joi_it");
/**
 * This class is used to validate an object using a given schema.
 * It is used by Lynx to automatically validate the body of any requests, using
 * the Body decodator.
 */
var ValidateObject = /** @class */ (function () {
    function ValidateObject(obj, schema, locales) {
        this._obj = obj;
        this.schema = schema;
        var options = null;
        for (var _i = 0, locales_1 = locales; _i < locales_1.length; _i++) {
            var locale = locales_1[_i];
            if (locale.indexOf("en") != -1) {
                break;
            }
            if (locale == "it") {
                options = {
                    language: itErrors.errors
                };
                break;
            }
        }
        this.validate(options);
    }
    ValidateObject.prototype.validate = function (options) {
        this.valid = Joi.validate(this._obj, this.schema, options);
    };
    Object.defineProperty(ValidateObject.prototype, "isValid", {
        /**
         * Verify that the object rescpect the schema.
         * @return true if the object is valid, false otherwise.
         */
        get: function () {
            return this.valid.error === null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ValidateObject.prototype, "obj", {
        /**
         * Unwrap the object (can be valid or not!)
         * @return the unwrapped object
         */
        get: function () {
            return this._obj;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ValidateObject.prototype, "errors", {
        /**
         * Getter that returns an array of validation errors.
         * @return an array of validation errors. It can not be null.
         */
        get: function () {
            var errors = [];
            for (var _i = 0, _a = this.valid.error.details; _i < _a.length; _i++) {
                var err = _a[_i];
                errors.push({
                    name: err.context && err.context.key ? err.context.key : "",
                    message: err.message
                });
            }
            return errors;
        },
        enumerable: true,
        configurable: true
    });
    return ValidateObject;
}());
exports.ValidateObject = ValidateObject;
