import { BaseController } from "lynx-framework/base.controller";
import Request from "lynx-framework/request";
import Response from "lynx-framework/lynx/response";
import { Route, GET, POST, Body, Name } from "lynx-framework/decorators";
import { ValidateObject } from "lynx-framework/validate-object";
import userLib from "lynx-framework/libs/users";
import User from "lynx-framework/entities/user.entity";

import AuthModule from "../index";

@Route(AuthModule.settings.controllerPath)
export default class AuthController extends BaseController {
    private get context() {
        let c: any = {};
        let original: any = AuthModule.settings.context as any;
        for (let key in original) {
            c[key] = original[key];
        }
        return c;
    }

    @GET("/login")
    async getLogin(req: Request): Promise<Response> {
        let c = this.context;
        return this.render(AuthModule.settings.loginTemplatePath, req, c);
    }

    @Name("auth_perform_login")
    @Body("d", AuthModule.settings.loginSchema)
    @POST("/login")
    async postLogin(
        d: ValidateObject<{ email: string; password: string }>,
        req: Request
    ): Promise<Response> {
        let c = this.context;
        if (!d.isValid) {
            c.errors = d.errors;
            return this.render(AuthModule.settings.loginTemplatePath, req, c);
        }
        let user = await userLib.performLogin(d.obj.email, d.obj.password);
        if (!(user instanceof User)) {
            return this.render(AuthModule.settings.loginTemplatePath, req, {
                loginError: true
            });
        }
        if (!user.isAdmin && !user.isStuff) {
            return this.unauthorized();
        }
        userLib.createUserSession(req, user);
        return this.redirect(AuthModule.settings.redirectPath);
    }
}
