"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var express = require("express");
var response_1 = require("./response");
var validate_object_1 = require("./validate-object");
var http_verb_1 = require("./http-verb");
function retrieveArgumentsNamesFromRoute(path) {
    var args = [];
    var parts = path.split("/");
    for (var _i = 0, parts_1 = parts; _i < parts_1.length; _i++) {
        var part = parts_1[_i];
        if (part.startsWith(":")) {
            var arg = part.substring(1, part.length);
            var endIndex = arg.indexOf("(");
            if (endIndex != -1) {
                arg = arg.substring(0, endIndex);
            }
            args.push(arg);
        }
    }
    return args;
}
function generateStandardCallback(controller, route) {
    var _this = this;
    return function (req, res, next) { return __awaiter(_this, void 0, void 0, function () {
        var _i, _a, verify, f, argsNames, argsValues, _b, argsNames_1, arg, b;
        return __generator(this, function (_c) {
            switch (_c.label) {
                case 0:
                    if (route.verifiers) {
                        for (_i = 0, _a = route.verifiers; _i < _a.length; _i++) {
                            verify = _a[_i];
                            if (!verify(req, res)) {
                                return [2 /*return*/, next()];
                            }
                        }
                    }
                    if (!!controller._hasBeenInit) return [3 /*break*/, 2];
                    return [4 /*yield*/, controller.postConstructor()];
                case 1:
                    _c.sent();
                    _c.label = 2;
                case 2:
                    f = controller[route.method];
                    argsNames = retrieveArgumentsNamesFromRoute(route.path);
                    argsValues = [];
                    for (_b = 0, argsNames_1 = argsNames; _b < argsNames_1.length; _b++) {
                        arg = argsNames_1[_b];
                        argsValues.push(req.params[arg]);
                    }
                    if (route.body != null) {
                        b = req.body;
                        if (route.body.schema) {
                            b = new validate_object_1.ValidateObject(b, route.body.schema, req.acceptsLanguages());
                        }
                        argsValues.push(b);
                    }
                    argsValues.push(req);
                    argsValues.push(res);
                    f
                        .apply(controller, argsValues)
                        .then(function (r) {
                        if (route.isAPI) {
                            if (typeof r === "boolean") {
                                return res.send({ success: r });
                            }
                            if (r.serialize) {
                                r = r.serialize();
                            }
                            return res.send({ success: true, data: r });
                        }
                        else {
                            if (r instanceof response_1.default) {
                                return r.performResponse(req, res);
                            }
                            res.send(r);
                        }
                    })
                        .catch(function (error) {
                        console.log(error);
                        if (route.isAPI) {
                            res
                                .status(400)
                                .send({ success: false, error: error.message });
                        }
                        else {
                            res.status(400).send(error);
                        }
                    });
                    return [2 /*return*/];
            }
        });
    }); };
}
function generateRouter(app, controller, originalController, routes) {
    var router = express.Router();
    for (var _i = 0, _a = originalController.routes; _i < _a.length; _i++) {
        var route = _a[_i];
        var func = void 0;
        switch (route.type) {
            case http_verb_1.HttpVerb.GET:
                func = router.get;
                break;
            case http_verb_1.HttpVerb.POST:
                func = router.post;
                break;
            case http_verb_1.HttpVerb.PUT:
                func = router.put;
                break;
            case http_verb_1.HttpVerb.DELETE:
                func = router.delete;
                break;
            case http_verb_1.HttpVerb.PATCH:
                func = router.patch;
                break;
            default:
                throw new Error("The decoration type for the method " +
                    route.method +
                    " is invalid");
        }
        if (route.name) {
            routes[route.name] = (originalController.controllerPath +
                "/" +
                route.path).replace(/\/\/+/g, "/");
        }
        var callback = generateStandardCallback(controller, route);
        if (route.isMultipartForm) {
            func.call(router, route.path, app.upload.any(), callback);
        }
        else {
            func.call(router, route.path, callback);
        }
    }
    return router;
}
exports.generateRouter = generateRouter;
function useController(app, Controller, routes) {
    if (!Controller.controllerPath) {
        throw new Error('You should decorate the "' +
            Controller.name +
            '" class in order to use it');
    }
    var controller = new Controller(app);
    controller._metadata = Controller;
    var router = generateRouter(app, controller, Controller, routes);
    app.express.use(Controller.controllerPath, router);
}
exports.useController = useController;
function generateMiddlewareCallback(middleware) {
    return function (req, res, next) {
        var f = middleware.apply;
        var argsValues = [];
        argsValues.push(req);
        argsValues.push(res);
        f
            .apply(middleware, argsValues)
            .then(function (_) {
            next();
        })
            .catch(function (error) {
            console.log(error);
            res.status(400).send(error);
        });
    };
}
function generateMiddlewares(app, middleware, path) {
    var callback = generateMiddlewareCallback(middleware);
    app.express.use(path, callback);
}
function useMiddleware(app, Middleware) {
    if (!Middleware.middlewarePath) {
        throw new Error('You should use at least one Middleware() decorator to a method of the "' +
            Middleware.name +
            '" class.');
    }
    var middleware = new Middleware(app);
    generateMiddlewares(app, middleware, Middleware.middlewarePath);
}
exports.useMiddleware = useMiddleware;
