/// <reference types="express" />
import { Request as ERequest, Response as EResponse } from "express";
import Response from "./response";
/**
 * This interface defines the currently available options for files.
 */
export interface FileOptions {
    width: number;
    height?: number;
}
/**
 * Generation of a File response.
 */
export default class FileResponse extends Response {
    private path;
    private _contentType?;
    private _options?;
    constructor(path: string);
    /**
     * Set the content type of the response.
     * @param value the content type
     */
    contentType: string;
    /**
     * Set the file options, to correctly generate the response.
     * @param value the FileOptions
     */
    options: FileOptions;
    /**
     * Generation of the response.
     * This method can eventually perform some transformation on output if a
     * FileOptions was specified.
     */
    performResponse(_: ERequest, res: EResponse): void;
}
