"use strict";
/**
 * Library containing useful function to generate and encrypt password and so on.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var bcrypt = require("bcrypt");
var _generatePassword = require("password-generator");
var user_entity_1 = require("../entities/user.entity");
/**
 * Encrypt a plain password. This method support async/await.
 * @param plainPassword the original plain text password
 * @return a promise with the ecrypted password
 */
function hashPassword(plainPassword) {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            return [2 /*return*/, bcrypt.hash(plainPassword, 10)];
        });
    });
}
exports.hashPassword = hashPassword;
/**
 * Verify that a plain password match with an encrypted hashPassword.
 * This method support async/await
 * @param plainPassword the plain text password
 * @param hashPassword the encrypted (or hashed) password
 * @return a primise, true if the two passwords match, false otherwise
 */
function verifyPassword(plainPassword, hashPassword) {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            return [2 /*return*/, bcrypt.compare(plainPassword, hashPassword)];
        });
    });
}
exports.verifyPassword = verifyPassword;
/**
 * Generate a new text plain password
 * @return a new generated password
 */
function generatePassword() {
    return _generatePassword(6);
}
exports.generatePassword = generatePassword;
/**
 *  Enum with self-explanatories errors for the user login method
 */
var UserErrors;
(function (UserErrors) {
    UserErrors[UserErrors["NOT_FOUND"] = 1] = "NOT_FOUND";
    UserErrors[UserErrors["WRONG_PASSWORD"] = 2] = "WRONG_PASSWORD";
})(UserErrors = exports.UserErrors || (exports.UserErrors = {}));
/**
 * Execute the login of a user, using the standard User Entity.
 * @param email the user email
 * @param password the plain text password
 * @return a promise, with an error if somethink goes wrong, or the user if the login was successul
 */
function performLogin(email, password) {
    return __awaiter(this, void 0, void 0, function () {
        var user;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, user_entity_1.default.findOne({ email: email })];
                case 1:
                    user = _a.sent();
                    if (!user) {
                        return [2 /*return*/, UserErrors.NOT_FOUND];
                    }
                    return [4 /*yield*/, verifyPassword(password, user.password)];
                case 2:
                    if (!(_a.sent())) {
                        return [2 /*return*/, UserErrors.WRONG_PASSWORD];
                    }
                    return [2 /*return*/, user];
            }
        });
    });
}
exports.performLogin = performLogin;
/**
 * Create a new cookie-based session for the user.
 * @param req the standard Express session
 * @param user the logged-in user
 */
function createUserSession(req, user) {
    req.session.user_id = user.id;
}
exports.createUserSession = createUserSession;
/**
 * Destoy (if any) the current user session.
 * @param req the standard Express session
 */
function destroyUserSession(req) {
    delete req.session.user_id;
}
exports.destroyUserSession = destroyUserSession;
/**
 * Retrieve the user from the current session
 * @param req the standard Express session
 * @return a primise, with the user if logged, or undefined
 */
function retrieveUserFromSession(req) {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            if (!req.session) {
                console.error("No connection to REDIS!!!!");
                process.exit(2);
            }
            return [2 /*return*/, user_entity_1.default.findOneById(req.session.user_id)];
        });
    });
}
exports.retrieveUserFromSession = retrieveUserFromSession;
